clc; format short g; clear all; close all;

 
% import data from Excel
X = importdata('Dataset.xlsx')


PhiHat = zeros(228,76); 
V      = PhiHat; 
LB     = PhiHat; 
UB     = PhiHat; 
W      = zeros(228,228,76);
reject = zeros(1,76);

% Perform estimation loop over all data vintages
for tau = 1:1:76;                                                          % loop over vintages (1=1997:Q1, ... , 67=2013:Q3, ..., 76=2015:Q4) 
    
    % define variables for the VAR 
    t0   = 20;                                                             % designate initial period (20=1964:Q3) ... includes 4 presample quarters!
    tT   = 148;                                                            % designate terminal period (144=1995:Q3, 148=1996:Q3, 149=1996:Q4)

    % sample size remains fixed 
    %----------------------------------
    y    = X.data.output(t0:tT,tau);                                       % y    = log of real GDP
    c    = X.data.consumption(t0:tT,tau);                                  % c    = log of real consumption   
    p    = X.data.priceLevel(t0:tT,tau);                                   % p    = log of price index
    i    = X.data.investment(t0:tT,tau);                                   % i    = log of real investment
    w    = X.data.realWage(t0:tT,tau);                                     % w    = log of real wage
    mpl  = X.data.laborProductivity(t0:tT,tau);                            % mpl  = log of labor productivity       
    ff   = X.data.fedFundsRate(t0:tT,tau);                                 % ff   = federal funds rate (APR)
    div  = X.data.realProfits(t0:tT,tau);                                  % div  = log of real profits   
    dm2  = X.data.moneyGrowth(t0:tT,tau);                                  % dm2  = growth rate of M2 (APR)   
    %----------------------------------

%     % sample size grows with vintage
%     %----------------------------------
%     y    = X.data.output(t0:tT-1+tau,tau);                                 % y    = log of real GDP
%     c    = X.data.consumption(t0:tT-1+tau,tau);                            % c    = log of real consumption   
%     p    = X.data.priceLevel(t0:tT-1+tau,tau);                             % p    = log of price index
%     i    = X.data.investment(t0:tT-1+tau,tau);                             % i    = log of real investment
%     w    = X.data.realWage(t0:tT-1+tau,tau);                               % w    = log of real wage
%     mpl  = X.data.laborProductivity(t0:tT-1+tau,tau);                      % mpl  = log of labor productivity       
%     ff   = X.data.fedFundsRate(t0:tT-1+tau,tau);                           % ff   = federal funds rate (APR)
%     div  = X.data.realProfits(t0:tT-1+tau,tau);                            % div  = log of real profits   
%     dm2  = X.data.moneyGrowth(t0:tT-1+tau,tau);                            % dm2  = growth rate of M2 (APR) 
%     %----------------------------------
    
    % Create a VAR(p) specification and estimate
    %----------------------------------------------------------------------
    Y = [y c p i w mpl ff div dm2/4];                                               % Y = data matrix

    Spec = vgxset('n',9,'nAR',4,'Constant',true);
    n = Spec.n; pl = Spec.nAR;

    [EstSpec,EstStdErrors,LLF,Resid] = vgxvarx(Spec,Y(pl+1:end,:),[],Y(1:pl,:));    % Warning! By default vgxvarx.m pads presample observations with zeros. 
    %----------------------------------------------------------------------
    
    % Apply recursive identification of monetary policy shocks
    %----------------------------------------------------------------------
    StructSpec  = recursiveness(EstSpec);
    DynRespFunc = impulse(EstSpec,StructSpec,7,25);
    %----------------------------------------------------------------------
    
    % Compute confidence intervals by simulating from a guassian distribution
    %----------------------------------------------------------------------
    alpha = 0.05; trials = 10000; burn = 2000;
    errorbands = irfbands_guassian(Y,EstSpec,DynRespFunc,alpha,trials,burn,1);
    reject(tau) = 100*errorbands.reject/trials;
    %----------------------------------------------------------------------
    
    % Compute dynamic response functions
    %----------------------------------------------------------------------
    irf_y    = -DynRespFunc.Phi{1}; 
    irf_c    = -DynRespFunc.Phi{2}; 
    irf_p    = -DynRespFunc.Phi{3}; 
    irf_i    = -DynRespFunc.Phi{4}; 
    irf_w    = -DynRespFunc.Phi{5}; 
    irf_mpl  = -DynRespFunc.Phi{6}; 
    irf_ff   = -DynRespFunc.Phi{7}; 
    irf_div  = -DynRespFunc.Phi{8}; 
    irf_dm2  = -DynRespFunc.Phi{9}; 
    irf_pi   = 4*(irf_p - [0 irf_p(1:end-1)]);

    % collect responses in vector PhiHat
    PhiHat(:,tau) = [irf_y(2:end) irf_c(2:end)   irf_pi(2:end) irf_i(2:end) ...
                     irf_w(2:end) irf_mpl(2:end) irf_ff        irf_div      irf_dm2]';
    %----------------------------------------------------------------------
    
    % Compute sample variances of the response functions
    %----------------------------------------------------------------------

    % sample distribution for irf_pi
    %-------------------------------
    count = errorbands.accept;
    Pi = zeros(1,26,count);
    for j = 1:1:count;
        Pi(:,:,j) = 4*(-errorbands.unsortedRF(3,:,j) - [0 -errorbands.unsortedRF(3,1:end-1,j)]);
    end;
    Pi    = sort(Pi,3,'ascend');
    lower = ceil((alpha/2)*count);
    upper = floor((1-(alpha/2))*count);
    Pi_LB = Pi(:,:,lower+1);
    Pi_UB = Pi(:,:,upper);
    %-------------------------------
    
    V_y      = errorbands.RespFuncs(1,:,:); V_y      = reshape(V_y,26,count)'; 
    V_yy     = diag(cov(V_y));              V_yy     = V_yy(2:end);

    V_c      = errorbands.RespFuncs(2,:,:); V_c      = reshape(V_c,26,count)'; 
    V_cc     = diag(cov(V_c));              V_cc     = V_cc(2:end);

    V_pi     = Pi;                          V_pi     = reshape(V_pi,26,count)';
    V_pipi   = diag(cov(V_pi));             V_pipi   = V_pipi(2:end);

    V_i      = errorbands.RespFuncs(4,:,:); V_i      = reshape(V_i,26,count)'; 
    V_ii     = diag(cov(V_i));              V_ii     = V_ii(2:end);

    V_w      = errorbands.RespFuncs(5,:,:); V_w      = reshape(V_w,26,count)'; 
    V_ww     = diag(cov(V_w));              V_ww     = V_ww(2:end);

    V_mpl    = errorbands.RespFuncs(6,:,:); V_mpl    = reshape(V_mpl,26,count)'; 
    V_mplmpl = diag(cov(V_mpl));            V_mplmpl = V_mplmpl(2:end);

    V_ff     = errorbands.RespFuncs(7,:,:); V_ff     = reshape(V_ff,26,count)'; 
    V_ffff   = diag(cov(V_ff));             V_ffff   = V_ffff(1:end);

    V_div    = errorbands.RespFuncs(8,:,:); V_div    = reshape(V_div,26,count)'; 
    V_divdiv = diag(cov(V_div));            V_divdiv = V_divdiv(1:end);

    V_dm2    = errorbands.RespFuncs(9,:,:); V_dm2    = reshape(V_dm2,26,count)'; 
    V_dm2dm2 = diag(cov(V_dm2));            V_dm2dm2 = V_dm2dm2(1:end);

    % collect variances in vector V
    V(:,tau) = [V_yy; V_cc; V_pipi; V_ii; V_ww; V_mplmpl; V_ffff; V_divdiv; V_dm2dm2];
    
    % construct sample covariance matrix
    Omega = [V_y(:,2:end) V_c(:,2:end) V_pi(:,2:end) V_i(:,2:end) V_w(:,2:end) V_mpl(:,2:end) V_ff V_div V_dm2];
    W(:,:,tau) = cov(Omega); 
    %----------------------------------------------------------------------
    
    % Collect the confidence bands
    %----------------------------------------------------------------------
    irf_y_UB    = -errorbands.LB{1}; irf_y_LB    = -errorbands.UB{1};
    irf_c_UB    = -errorbands.LB{2}; irf_c_LB    = -errorbands.UB{2};
    irf_pi_UB   = Pi_UB;             irf_pi_LB   = Pi_LB;
    irf_i_UB    = -errorbands.LB{4}; irf_i_LB    = -errorbands.UB{4};
    irf_w_UB    = -errorbands.LB{5}; irf_w_LB    = -errorbands.UB{5};
    irf_mpl_UB  = -errorbands.LB{6}; irf_mpl_LB  = -errorbands.UB{6};
    irf_ff_UB   = -errorbands.LB{7}; irf_ff_LB   = -errorbands.UB{7};
    irf_div_UB  = -errorbands.LB{8}; irf_div_LB  = -errorbands.UB{8};
    irf_dm2_UB  = -errorbands.LB{9}; irf_dm2_LB  = -errorbands.UB{9};


    LB(:,tau) = [irf_y_LB(2:end) irf_c_LB(2:end) irf_pi_LB(2:end) irf_i_LB(2:end) irf_w_LB(2:end) irf_mpl_LB(2:end) irf_ff_LB irf_div_LB irf_dm2_LB]';
    UB(:,tau) = [irf_y_UB(2:end) irf_c_UB(2:end) irf_pi_UB(2:end) irf_i_UB(2:end) irf_w_UB(2:end) irf_mpl_UB(2:end) irf_ff_UB irf_div_UB irf_dm2_UB]';
    %----------------------------------------------------------------------
    
end;


% sample size remains fixed 
%--------------------------
save PhiHat PhiHat
save V V 
save LB LB
save UB UB
save W W
save reject reject
%--------------------------


% % sample size grows with vintage 
% %--------------------------
% PhiHat_s = PhiHat; V_s = V; LB_s = LB; UB_s = UB; W_s = W; reject_s = reject;
% save PhiHat_s PhiHat_s
% save V_s V_s 
% save LB_s LB_s
% save UB_s UB_s
% save W_s W_s
% save reject_s reject_s
% %--------------------------



    
    
    
    
    
    
    
    