clc; format long g; clear all; close all;


% Load structural parameter estimates
X = importdata('Estimation Results.xlsx')

% Load PhiHat, V, and W matrices for estimation
load PhiHat
load V
load W

SE = zeros(7,76);
for tau = 1:1:76;                                                          % designate vintage period (1=1997:Q1, 65=2013:Q1, 67=2013:Q3, 76=2015:Q4)
    
phiHat    = PhiHat(:,tau); 
v         = V(:,tau); 
w         = W(:,:,tau);
param_opt = X.data.estimates(1:7,tau)


% Compute parameter standard errors using the delta function method
%--------------------------------------------------------------------------
% obtain J11
output = grad_hess('irf_criterion',param_opt,1e-5,phiHat,v);
J11    = output.hessian;

% obtain J12
tic
J12    = deriv12_irf_criterion(phiHat,1e-5,param_opt,v);
toc

% approximate asymptotic covariance matrix
Sigma      = (-J11\J12)*w*(-J11\J12)';
std_errors = sqrt(diag(Sigma));
%--------------------------------------------------------------------------

SE(:,tau) = std_errors;
end;


        
        
        
        
        