function DynRespFunc = impulse(EstSpec,StructSpec,k,T);
%--------------------------------------------------------------------------
% PURPOSE: computes orthogonalized dynamic response functions for a VAR         
%--------------------------------------------------------------------------
% USAGE:   DynRespFunc = impulse(EstSpec,StructSpec,k,T) 
% where:   EstSpec     = a vgxvarx.m output 'structure'
%          StructSpec  = a recursiveness.m output 'structure'
%          k           = kth element of the structural error vector
%          T           = response function horizon
%--------------------------------------------------------------------------
% RETURNS a 'structure'
% DynRespFunc.Phi      = (1xT) response profile for each variable in VAR 
%--------------------------------------------------------------------------
% written by: Gregory Givens
%             27 February 2013
%--------------------------------------------------------------------------


DynRespFunc.method = 'dynamic responses to an orthogonalized shock';

n = EstSpec.n;      q = EstSpec.nAR;
A0 = StructSpec.A0; D = StructSpec.D;

for i = 1:1:q;
   A(:,:,i) = EstSpec.AR{i};
end;
A = reshape(A,n,n*q);
B = [A; eye(n*(q-1)) zeros(n*(q-1),n)];     % builds companion matrix

select = [eye(n) zeros(n,n*(q-1))];
IRF = zeros(n,T+1);    
for j = 0:1:T;
    IRF(:,j+1) = select*(B^j)*[inv(A0); zeros(n*(q-1),n)]*diag(e(k,n))*diag(sqrt(D));
end;

for h = 1:1:n;
   Phi{h} = IRF(h,:);
end;

DynRespFunc.horizon = T;
DynRespFunc.shock   = k;
DynRespFunc.Phi     = Phi;