function [J,param_opt,Phi] = irf_criterion(param,PhiHat,V);



% Model parameters
%--------------------------------------------------------------------------
% 1st group (calibrated): 
beta     = 1.03^(-0.25);
alpha    = 0.36;
delta    = 0.025;
QM       = 0.44;
mu       = 1.017;
lambda_w = 1.05;

% 2nd group (monetary policy):
rho0 = 0; rho1 = 0; a = PhiHat(203:1:228);

% 3rd group (estimated):
lambda_f = param(1);
eps_w    = param(2);
eps_p    = param(3);
sigma_q  = param(4);
kappa    = param(5);
b        = param(6);
sigma_a  = param(7); 
%sigma_a = 1000;

% 4th group (auxiliary):
R        = mu/beta;
rk       = (1/beta) - (1-delta);
KY       = alpha/rk;
CY       = 1 - (alpha*delta/rk);
q_w      = mu/(mu-QM);
b_w      = (2*lambda_w-1)/((1-eps_w)*(1-beta*eps_w));
%--------------------------------------------------------------------------


% Coefficient matrices
%--------------------------------------------------------------------------
n1 = 43; n2 = 19; n = n1 + n2; e0 = zeros(1,n);

A = [e(1,n); e(2,n); ...
    
    e(3,n); e(4,n); e(5,n); e(6,n); e(7,n); e(8,n); e(9,n); e(10,n); e(11,n); e(12,n); e(13,n); e(14,n); e(15,n); ...
    e(16,n); e(17,n); e(18,n); e(19,n); e(20,n); e(21,n); e(22,n); e(23,n); e(24,n); e(25,n); e(26,n); e(27,n);  ...
    
    e(28,n); e(29,n); e(30,n); e(31,n); e(32,n); e(33,n); e(34,n); e(35,n)-e(57,n); e(36,n); e(37,n)-e(56,n); ...
    e(38,n)-e(58,n); e(39,n)-e(59,n); e(40,n)-e(60,n); e(41,n)-e(61,n); e(42,n)-e(62,n); e(43,n)-e(55,n); ...
      
    e0; e(45,n) + e(46,n) - e(47,n); e0; e0; e0; e0; e0; e0; e0; e0; e0; e0; ...
    e(44,n); e(47,n); e(51,n); e(52,n); e(53,n); e(45,n); e(48,n)];    
        
MvgAvg = zeros(1,n);
for i = 1:25;
    MvgAvg = MvgAvg + a(i+1)*e(i+2,n);
end;

B = [rho0*e(1,n) + rho1*e(2,n) + MvgAvg; e(1,n); ...
    
    e0; e(3,n); e(4,n); e(5,n); e(6,n); e(7,n); e(8,n); e(9,n); e(10,n); e(11,n); e(12,n); e(13,n); e(14,n); ...
    e(15,n); e(16,n); e(17,n); e(18,n); e(19,n); e(20,n); e(21,n); e(22,n); e(23,n); e(24,n); e(25,n); e(26,n); ...
        
    e(2,n) + e(28,n) - e(47,n); e(44,n); e(52,n); (1-delta)*e(31,n) + delta*e(52,n); e(47,n); ...
    e(53,n); e(56,n); e0; e(58,n); e0; e0; e0; e0; e0; e0; e0; ...
    
    (1+beta*b^2)*e(44,n) - b*e(29,n) - beta*b*e(35,n) + (1-b)*(1-beta*b)*e(34,n); ...
    e(45,n); ...
    sigma_a*e(49,n) - e(36,n); ... 
    e(50,n) + (1/sigma_q)*((R/(R-1))*e(46,n)+e(45,n)); ...
    e(51,n) + e(34,n) - e(37,n) - (1-beta*(1-delta))*e(38,n) - beta*(1-delta)*e(39,n); ...
    e(51,n) - kappa*(e(52,n)-e(30,n)) + beta*kappa*e(40,n) - beta*kappa*e(52,n); ...
    -e(48,n) + e(53,n) + e(46,n) + e(54,n) - (e(49,n)+e(31,n)); ...
    -e(55,n) + alpha*e(48,n) + (1-alpha)*(e(46,n)+e(53,n)); ...
    -lambda_f*(alpha*(e(49,n)+e(31,n)) + (1-alpha)*e(54,n)) + CY*e(44,n) + delta*KY*e(52,n) + ((1/beta)-(1-delta))*KY*e(49,n); ...
    -e(53,n) - e(54,n) + q_w*(e(1,n)+e(2,n)+e(28,n)-e(47,n)) + (1-q_w)*e(50,n); ...
    -e(47,n) + (1/(1+beta))*e(32,n) + (beta/(1+beta))*e(41,n) + (((1-eps_p)*(1-beta*eps_p))/((1+beta)*eps_p))*e(43,n); ...
    -((b_w*(1+beta*eps_w^2)-lambda_w)/(b_w*eps_w))*e(53,n) + e(33,n) - (1+beta)*e(47,n) + e(32,n) + beta*(e(41,n)+e(42,n)) + ((1-lambda_w)/(b_w*eps_w))*(e(34,n)-e(54,n)); ...
    e(57,n); e(61,n); e(59,n); e(60,n); e(62,n); e(56,n); e(58,n)];    
%--------------------------------------------------------------------------


% Rational expectations equilibrium
%--------------------------------------------------------------------------
[M,C,numb] = klein(A,B,n1,n2);

N = [a(1) zeros(1,n1-1); zeros(1,n1); e(1,n1); zeros(n1-3,n1)];
%--------------------------------------------------------------------------


% Compute impulse response functions
%--------------------------------------------------------------------------
T = 25;

shock = [1 0 1 zeros(1,n1-3)]';
irf_X = zeros(size(M,1),T+1);
irf_x = zeros(size(C,1),T+1);

for j = 1:1:T+1;
    irf_X(:,j) = M^(j-1)*N*shock;
    irf_x(:,j) = C*M^(j-1)*N*shock;
end;
%--------------------------------------------------------------------------


% Collect impulse response functions 
%--------------------------------------------------------------------------
consumption  = irf_x(1,2:end);
investment   = irf_x(9,2:end);
utilization  = irf_x(6,2:end);
output       = CY*consumption + delta*KY*investment + ((1/beta)-(1-delta))*KY*utilization;
hours        = irf_x(11,2:end);
productivity = output - hours;
inflation    = irf_x(4,2:end)*4;
wage         = irf_x(10,2:end);
interest     = irf_x(3,:)*4;
moneygrowth  = irf_X(1,:);
marginalcost = irf_x(12,:);
profits      = [0 output] - (1/(lambda_f-1))*marginalcost;

Phi = [output consumption inflation investment wage productivity interest profits moneygrowth]';
%--------------------------------------------------------------------------


% Compute estimation criterion
%--------------------------------------------------------------------------
J = ((PhiHat - Phi)'/diag(V))*(PhiHat - Phi);
param_opt = param;
%--------------------------------------------------------------------------






