clc; format long g; clear all; close all;



tau = 76;                                                                  % designate vintage period (1=1997:Q1, 41=2007:Q1, 61=2012:Q1, 65=2013:Q1, 67=2013:Q3, 76=2015:Q4)


% Unpack structural parameter estimates and standard errors
X = importdata('Estimation Results.xlsx')

lambda_f = X.data.estimates(1,1:tau); se1 = X.data.estimates(13,1:tau);
eps_w    = X.data.estimates(2,1:tau); se2 = X.data.estimates(14,1:tau);
eps_p    = X.data.estimates(3,1:tau); se3 = X.data.estimates(15,1:tau);
sigma_q  = X.data.estimates(4,1:tau); se4 = X.data.estimates(16,1:tau);
kappa    = X.data.estimates(5,1:tau); se5 = X.data.estimates(17,1:tau);
b        = X.data.estimates(6,1:tau); se6 = X.data.estimates(18,1:tau);
sigma_a  = X.data.estimates(7,1:tau); se7 = X.data.estimates(19,1:tau);
J        = X.data.estimates(9,1:tau);


% Compute Wald statistics for each parameter across all vintages
Chi2_1 = (lambda_f.^2)./(se1.^2); Pval_1 = 1 - cdf('chi2',Chi2_1,1);
Chi2_2 = (eps_w.^2)./(se2.^2);    Pval_2 = 1 - cdf('chi2',Chi2_2,1);
Chi2_3 = (eps_p.^2)./(se3.^2);    Pval_3 = 1 - cdf('chi2',Chi2_3,1);
Chi2_4 = (sigma_q.^2)./(se4.^2);  Pval_4 = 1 - cdf('chi2',Chi2_4,1);
Chi2_5 = (kappa.^2)./(se5.^2);    Pval_5 = 1 - cdf('chi2',Chi2_5,1);
Chi2_6 = (b.^2)./(se6.^2);        Pval_6 = 1 - cdf('chi2',Chi2_6,1);
Chi2_7 = (sigma_a.^2)./(se7.^2);  Pval_7 = 1 - cdf('chi2',Chi2_7,1);


% Find dates where parameters are insignificant
index1 = find(Pval_1 >= 0.05);
index2 = find(Pval_2 >= 0.05);
index3 = find(Pval_3 >= 0.05);
index4 = find(Pval_4 >= 0.05);
index5 = find(Pval_5 >= 0.05);
index6 = find(Pval_6 >= 0.05);
index7 = find(Pval_7 >= 0.05);


% Percentage of vintages that yield statistically significant estimates
P = (1 - [length(index1) length(index2) length(index3) length(index4) length(index5) length(index6) length(index7)]./tau).*100; 


% Compute summary statistics across vintages
display('mean min max SD mean(s.e.) SD(s.e.) % = ')
[mean(lambda_f) min(lambda_f) max(lambda_f) std(lambda_f) mean(se1) std(se1) P(1);
 mean(eps_w)    min(eps_w)    max(eps_w)    std(eps_w)    mean(se2) std(se2) P(2);   
 mean(eps_p)    min(eps_p)    max(eps_p)    std(eps_p)    mean(se3) std(se3) P(3);
 mean(sigma_q)  min(sigma_q)  max(sigma_q)  std(sigma_q)  mean(se4) std(se4) P(4);
 mean(kappa)    min(kappa)    max(kappa)    std(kappa)    mean(se5) std(se5) P(5);
 mean(b)        min(b)        max(b)        std(b)        mean(se6) std(se6) P(6);
 mean(sigma_a)  min(sigma_a)  max(sigma_a)  std(sigma_a)  mean(se7) std(se7) P(7)] 


% Compute correlation matrix for parameter estimates
CorrMatrix = corr([lambda_f' eps_w' eps_p' sigma_q' kappa' b' sigma_a']);


% Plot estimates and standard errors across all vintages
time = 1997:0.25:(1997+0.25*(tau-1));

figure,
subplot(4,2,1)
[AX,H1,H2] = plotyy(time,lambda_f,time,se1); title('$$\lambda_f$$','Interpreter','LaTeX'), 
line(time(index1),lambda_f(index1),'Marker','o','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',3)
set(AX(1),'YLim',[1.0 1.4],'XLim',[1997 2015.75]), set(AX(1),'YTick',[1.0:0.1:1.4]) 
set(AX(2),'YLim',[0.0 1.0],'XLim',[1997 2015.75]), set(AX(2),'YTick',[0.0:0.25:1.0]) 
set(AX,{'ycolor'},{'k';'k'}), set(AX(1),'box','off')
set(H1,'LineStyle','-','Color','k'), set(H2,'LineStyle',':','Color','k')

subplot(4,2,2)
[AX,H1,H2] = plotyy(time,eps_w,time,se2); title('$$\xi_w$$','Interpreter','LaTeX'),  
set(AX(1),'YLim',[0.6 0.9],'XLim',[1997 2015.75]), set(AX(1),'YTick',[0.6:0.1:0.9]) 
set(AX(2),'YLim',[0.0 0.6],'XLim',[1997 2015.75]), set(AX(2),'YTick',[0.0:0.2:0.6]) 
set(AX,{'ycolor'},{'k';'k'}), set(AX(1),'box','off')
set(H1,'LineStyle','-','Color','k'), set(H2,'LineStyle',':','Color','k')

subplot(4,2,3)
[AX,H1,H2] = plotyy(time,eps_p,time,se3); title('$$\xi_p$$','Interpreter','LaTeX'), 
set(AX(1),'YLim',[0.4 0.7],'XLim',[1997 2015.75]), set(AX(1),'YTick',[0.4:0.1:0.7]) 
set(AX(2),'YLim',[0.0 0.4],'XLim',[1997 2015.75]), set(AX(2),'YTick',[0.0:0.1:0.4]) 
set(AX,{'ycolor'},{'k';'k'}), set(AX(1),'box','off')
set(H1,'LineStyle','-','Color','k'), set(H2,'LineStyle',':','Color','k')

subplot(4,2,4)
[AX,H1,H2] = plotyy(time,sigma_q,time,se4); title('$$\sigma_q$$','Interpreter','LaTeX'),  
line(time(index4),sigma_q(index4),'Marker','o','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',3)           
set(AX(1),'YLim',[15 24],'XLim',[1997 2015.75]), set(AX(1),'YTick',[15:3:24]) 
set(AX(2),'YLim',[5 26],'XLim',[1997 2015.75]), set(AX(2),'YTick',[5:5:26]) 
set(AX,{'ycolor'},{'k';'k'}), set(AX(1),'box','off')
set(H1,'LineStyle','-','Color','k'), set(H2,'LineStyle',':','Color','k')

subplot(4,2,5)
[AX,H1,H2] = plotyy(time,kappa,time,se5); title('$$\kappa$$','Interpreter','LaTeX'), 
line(time(index5),kappa(index5),'Marker','o','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',3)
set(AX(1),'YLim',[1 5],'XLim',[1997 2015.75]), set(AX(1),'YTick',[1:1:5]) 
set(AX(2),'YLim',[0 4],'XLim',[1997 2015.75]), set(AX(2),'YTick',[0:1:4]) 
set(AX,{'ycolor'},{'k';'k'}), set(AX(1),'box','off')
set(H1,'LineStyle','-','Color','k'), set(H2,'LineStyle',':','Color','k')

subplot(4,2,6)
[AX,H1,H2] = plotyy(time,b,time,se6); title('$$b$$','Interpreter','LaTeX'); 
set(AX(1),'YLim',[0.4 0.8],'XLim',[1997 2015.75]), set(AX(1),'YTick',[0.4:0.1:0.8]) 
set(AX(2),'YLim',[0.05 0.5],'XLim',[1997 2015.75]), set(AX(2),'YTick',[0:0.1:0.5]) 
set(AX,{'ycolor'},{'k';'k'}), set(AX(1),'box','off')
set(H1,'LineStyle','-','Color','k'), set(H2,'LineStyle',':','Color','k')

subplot(4,2,7)
[AX,H1,H2] = plotyy(time,sigma_a,time,se7); title('$$\sigma_a$$','Interpreter','LaTeX'), 
line(time(index7),sigma_a(index7),'Marker','o','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',3)
set(AX(1),'YLim',[0 0.8],'XLim',[1997 2015.75]), set(AX(1),'YTick',[0:0.2:0.8]) 
set(AX(2),'YLim',[0 6],'XLim',[1997 2015.75]), set(AX(2),'YTick',[0:2:6]) 
set(AX,{'ycolor'},{'k';'k'}), set(AX(1),'box','off')
set(H1,'LineStyle','-','Color','k'), set(H2,'LineStyle',':','Color','k')


% Analysis of absolute changes in structural parameter estimates
d1 = abs(lambda_f(2:end) - lambda_f(1:end-1)); [Y1, I1] = sort(d1,2,'descend'); dates1 = I1(1:3) + 1; values1 = Y1(1:3);
d2 = abs(eps_w(2:end)    - eps_w(1:end-1));    [Y2, I2] = sort(d2,2,'descend'); dates2 = I2(1:3) + 1; values2 = Y2(1:3);
d3 = abs(eps_p(2:end)    - eps_p(1:end-1));    [Y3, I3] = sort(d3,2,'descend'); dates3 = I3(1:3) + 1; values3 = Y3(1:3);
d4 = abs(sigma_q(2:end)  - sigma_q(1:end-1));  [Y4, I4] = sort(d4,2,'descend'); dates4 = I4(1:3) + 1; values4 = Y4(1:3);
d5 = abs(kappa(2:end)    - kappa(1:end-1));    [Y5, I5] = sort(d5,2,'descend'); dates5 = I5(1:3) + 1; values5 = Y5(1:3);
d6 = abs(b(2:end)        - b(1:end-1));        [Y6, I6] = sort(d6,2,'descend'); dates6 = I6(1:3) + 1; values6 = Y6(1:3);
d7 = abs(sigma_a(2:end)  - sigma_a(1:end-1));  [Y7, I7] = sort(d7,2,'descend'); dates7 = I7(1:3) + 1; values7 = Y7(1:3);

    
display('dates of 3 biggest absolute parameter changes are ... ')
[time(dates1); values1; time(dates2); values2; time(dates3); values3; time(dates4); values4; time(dates5); values5; time(dates6); values6;  time(dates7); values7]




